#Include "bass.bi"
#Include "bassmidi.bi"

Type MUSIC
	Declare Static Function Init( ByVal MusicFont As String ) As Integer
	Declare Static Sub Quit( )
	Declare Static Sub UpdateAll( )
	Declare Static Function create() As MUSIC Ptr
	Declare Sub load( ByVal File As String, ByVal looping As Integer )
	Declare Sub play()
	Declare Sub stop()
	Declare Sub destroy()
	Declare Sub destroy( ByVal _time As Single )
	Declare Sub setChannelVol( ByVal channel As Integer, ByVal vol As Single )
	Declare Sub setVol( ByVal vol As Single )
	Declare Sub slideVol( ByVal vol As Single, ByVal _time As Single )
	Declare Sub setPos( ByVal _pos As Single)
	Declare Function getPos()As Single
	Declare Sub update()
	_NextMusic As MUSIC Ptr
	_PrevMusic As MUSIC Ptr
	Private:
	_Music As HSTREAM
	_ChannelVol(0 To 15) As Single
	_MasterVol As Single
	_NewMasterVol As Single
	_ChangeTime As Single
	_ChangeTimeStart As Double
	_LastUpdateTime As Double
	_destroy As Integer
End Type

Type _MusicParam
	BassMidi As HPLUGIN
	BassMidiFont As Long
	FirstMusic As MUSIC Ptr
End Type

Dim Shared As _MusicParam MusicParam

Static _
Function MUSIC.Init( ByVal MusicFont As String ) As Integer
	InLog("BASS_Version: "+Str(HiByte(BASS_GetVersion()))+"."+Str(LoByte(BASS_GetVersion())))
	If( BASS_GetVersion( ) < MAKELONG(2,2) ) Then
		ErLog("Bass Version must be higher or 2.2")
		Return 0
	EndIf
	If( BASS_Init( -1, 44100, 0, 0, 0 ) = 0 ) Then
		ErLog("Bass Init failed")
		Return 0
	EndIf
	MusicParam.BassMidi = BASS_PluginLoad( ExePath+"/"+"bassmidi.dll" )
	If MusicParam.BassMidi = 0 Then
		ErLog("Bassmidi Init failed")
		Return 0
	EndIf
	MusicParam.BassMidiFont = BASS_MIDI_FontInit( ExePath+"/"+MusicFont, 0 )
	If MusicParam.BassMidiFont = 0 Then
		ErLog("Bassmidi Font load failed")
		Return 0
	EndIf
	Return -1
End Function
Static _
Sub MUSIC.Quit( )
	BASS_Stop( )
	BASS_MIDI_FontFree( MusicParam.BassMidiFont )
	BASS_PluginFree( MusicParam.BassMidi )
	BASS_Free( )
End Sub

Static _
Sub MUSIC.UpdateAll( )
	Dim As MUSIC Ptr _music,_nextmusic
	_music = MusicParam.FirstMusic
	Print _music
	Do Until _music=0
		_nextmusic=_music->_NextMusic
		_music->update()
		_music=_nextmusic
	Loop
End Sub
Static _
Function MUSIC.create() As MUSIC Ptr
	Dim As MUSIC Ptr _music
	_music = Allocate(SizeOf(MUSIC))
	For i As Integer = 0 To SizeOf(MUSIC)-1
		Cast(UByte Ptr, _music)[i] = 0
	Next
	For i As Integer = 0 To 15
		_music->setChannelVol(i,1)
	Next
	_music->setVol(1)
	_music->_NextMusic = MusicParam.FirstMusic
	If MusicParam.FirstMusic Then
		MusicParam.FirstMusic->_PrevMusic = _music
	EndIf
	MusicParam.FirstMusic=_music
	Return _music
End Function
Sub MUSIC.load( ByVal File As String, ByVal looping As Integer )
	Dim As BASS_MIDI_FONT Musicfont
	If This._Music Then
		This.stop()
		BASS_MusicFree( This._Music )
	EndIf
	InLog("Midi opened"""+File+"""")
	If looping Then
		This._Music = BASS_MIDI_StreamCreateFile( FALSE, StrPtr(File), 0, 0, BASS_MUSIC_LOOP, 0 )
	Else
		This._Music =  BASS_MIDI_StreamCreateFile( FALSE, StrPtr(File), 0, 0, 0, 0 )
	EndIf
	If This._Music = 0 Then
		ErLog("Midi opened failed """+File+"""")
	EndIf
	Musicfont.font = MusicParam.BassMidiFont
	Musicfont.presetno = -1
	Musicfont.bank = 0
	BASS_MIDI_StreamSetFonts(This._Music,@Musicfont,1)
End Sub
Sub MUSIC.play()
	BASS_ChannelPlay( This._Music, FALSE )
End Sub
Sub MUSIC.stop()
	BASS_ChannelStop( This._Music )
End Sub
Sub MUSIC.destroy()
	Dim toCill As MUSIC Ptr
	This.stop()
	BASS_MusicFree( This._Music )
	If This._NextMusic Then
		This._NextMusic->_PrevMusic = This._PrevMusic
	EndIf
	If This._PrevMusic Then
		This._PrevMusic->_NextMusic = This._NextMusic
	Else
		MusicParam.FirstMusic = This._NextMusic
	EndIf
	DeAllocate(@This)
End Sub
Sub MUSIC.destroy( ByVal _time As Single )
	This.slideVol(0,_time)
	This._destroy = -1
End Sub
Sub MUSIC.setChannelVol( ByVal channel As Integer, ByVal vol As Single )
	This._ChannelVol(channel) = vol
End Sub
Sub MUSIC.setVol( ByVal vol As Single )
	This._MasterVol = vol
	This._NewMasterVol = vol
	This._ChangeTime = -1
	This._ChangeTimeStart = -1
End Sub
Sub MUSIC.slideVol( ByVal vol As Single, ByVal _time As Single )
	Dim As Single MusicVol, prozent
	MusicVol = This._MasterVol
	If This._ChangeTime > 0 Then
		prozent = (This._LastUpdateTime - This._ChangeTimeStart)/This._ChangeTime
		If prozent < 0 Then prozent = 0
		If prozent > 1 Then prozent = 1
		MusicVol = This._MasterVol*(1-prozent)+This._NewMasterVol*prozent
	EndIf
	This._MasterVol = MusicVol
	This._NewMasterVol = vol
	This._ChangeTime = _time
	This._ChangeTimeStart = Timer
End Sub
Sub MUSIC.setPos( ByVal _pos As Single)
	Dim As QWORD length
	length = BASS_ChannelGetLength(This._Music)
	BASS_ChannelSetPosition(This._Music,_pos*length/255)
End Sub
Function MUSIC.getPos()As Single
	Dim As QWORD length
	Dim As QWORD _pos
	length = BASS_ChannelGetLength(This._Music)
	_pos = BASS_ChannelGetPosition(This._Music)
	Return _pos/length*255
End Function
Sub MUSIC.update()
	Dim As Single MusicVol, prozent
	Dim As Double t=Timer
	MusicVol = This._MasterVol
	If This._ChangeTime > 0 Then
		prozent = (This._LastUpdateTime - This._ChangeTimeStart)/This._ChangeTime
		If prozent < 0 Then prozent = 0
		If prozent > 1 Then prozent = 1
		MusicVol = This._MasterVol*(1-prozent)+This._NewMasterVol*prozent
		If prozent = 1 Then
			This._MasterVol = This._NewMasterVol
			This._ChangeTime = -1
			This._ChangeTimeStart = -1
			If This._destroy Then
				This.destroy()
				Exit Sub
			EndIf
		EndIf
	EndIf
	This._LastUpdateTime = t
	For i As Integer = 0 To 15
		BASS_MIDI_StreamEvent( This._Music, i, MIDI_EVENT_VOLUME, This._ChannelVol(i)*MusicVol*100 )
	Next
End Sub
