Type TEXTURE
	Declare Sub Bind()
	Declare Sub UnBind()
	Declare Sub Load ( ByVal As String, ByVal As UInteger = 0 )
	Declare Sub Destroy()
	Declare Function GetID() As UInteger
	Private:
	Declare Function LoadBMP ( ByVal As String, ByRef As Integer, ByRef As Integer, ByRef As Integer ) As Any Ptr
	Declare Function LoadCompressedTGA ( ByVal As String, ByRef As Integer, ByRef As Integer, ByRef As Integer ) As Any Ptr
	Declare Function LoadUncompressedTGA ( ByVal As String, ByRef As Integer, ByRef As Integer, ByRef As Integer ) As Any Ptr
	Declare Function MaskPic ( ByVal As UInteger, ByVal As Integer, ByVal As Integer, ByRef As Integer, ByVal As Any Ptr ) As Any Ptr
	Declare Function ArraySame ( ByVal As Any ptr, ByVal As Any Ptr, ByVal As Long) As Long
	ID As UInteger
End Type

Sub TEXTURE.Bind()
	glBindTexture(GL_TEXTURE_2D,This.ID)
End Sub
Sub TEXTURE.UnBind()
	glBindTexture(GL_TEXTURE_2D,0)
End Sub
Sub TEXTURE.Load ( ByVal File As String, ByVal Mask As UInteger = 0 )
	Const BITMAP_ID = &H4D42
	Dim BITMAP_ID_G As UShort
	Dim As Integer FF,Weite,Hohe,BBP
	Dim Z As Any Ptr
	Z = This.LoadBMP(File,Weite,Hohe,BBP)
	If Z Then
		If Mask <> 0 Then Z = This.MaskPic(Mask,Weite,Hohe,BBP,Z)
	Else
		Z = This.LoadCompressedTGA(File,Weite,Hohe,BBP)
	EndIf
	If Z=0 Then
		Z = This.LoadUncompressedTGA(File,Weite,Hohe,BBP)
	EndIf
	If Z = 0 Then
		Exit Sub
	EndIf
	glGenTextures(1,@This.ID)
	This.Bind()
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT)
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT)
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR)
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER ,GL_LINEAR)
	glTexImage2D(GL_TEXTURE_2D,0,BBP,Weite,Hohe,0,BBP,GL_UNSIGNED_BYTE,Z)
	This.UnBind()
	If Z Then DeAllocate(Z)
End Sub
Sub TEXTURE.Destroy()
	If This.ID Then glDeleteTextures(1,@This.ID)
	This.ID = 0
End Sub
Function TEXTURE.GetID()As UInteger
	Return This.ID
End Function
Function TEXTURE.LoadBMP ( ByVal File As String, ByRef W As Integer, ByRef H As Integer, ByRef BBP As Integer ) As Any Ptr
	Const BITMAP_ID = &H4D42
	
	Type BITMAPINFOHEADER Field = 1
		biSize As Integer
		biWidth As Integer
		biHeight As Integer
		biPlanes As Short
		biBitCount As Short
		biCompression As Integer
		biSizeImage As Integer
		biXPelsPerMeter As Integer
		biYPelsPerMeter As Integer
		biClrUsed As Integer
		biClrImportant As Integer
	End Type
	
	Type PALETTEENTRY
		peRed As Byte
		peGreen As Byte
		peBlue As Byte
		peFlags As Byte
	End Type
	
	Type BITMAPFILEHEADER Field = 1
		bfType As short
		bfSize As Integer
		bfReserved1 As Short
		bfReserved2 As Short
		bfOffBits As Integer
	End Type

	Dim bitmapfileheader as BITMAPFILEHEADER
  	Dim bitmapinfoheader as BITMAPINFOHEADER
  	Dim bmpalette(256) as PALETTEENTRY

   Dim index As Integer
   Dim noofpixels As Integer
   Dim p As UByte Ptr
   Dim As UByte r, g, b
   Dim ret As Any Ptr
   dim FF As Integer
 	FF = freefile
  
	If Open (File For Binary As #FF) <> 0 Then
		ErLog("File "+Chr(34)+File+Chr(34)+" not found")
		Return 0
	EndIf
  
   Get #FF, , bitmapfileheader
   If bitmapfileheader.bfType <> BITMAP_ID then
     'ErLog("Unknown File Format from File "+Chr(34)+File+Chr(34))
     Close #FF
     Return 0
   End if

   Get #FF, , bitmapinfoheader
	
   If bitmapinfoheader.biBitCount = 8 Or bitmapinfoheader.biBitCount = 24 Then
		
		BBP = GL_RGB
		
      If bitmapinfoheader.biBitCount = 8 Then
        	For index = 0 To 255
          	Get #FF, , bmpalette(index)
        	Next
      EndIf

      noofpixels = bitmapinfoheader.biWidth*bitmapinfoheader.biHeight
     
      W  = bitmapinfoheader.biWidth
      H  = bitmapinfoheader.biHeight
      ret = Allocate(noofpixels*3)
      If ret = 0 Then
        	ErLog("Allocate failed")
        	Close #FF
        	DeAllocate (ret)
        	Return 0
      EndIf

      p = ret
      If bitmapinfoheader.biBitCount=24 Then
        For index = 0 To noofpixels -1
          Get #FF, , b
          Get #FF, , g
          Get #FF, , r
          *p = r : p += 1
          *p = g : p += 1
          *p = b : p += 1
        Next
      Else
        For index = 0 To noofpixels -1
          Get #FF, , b
          *p = bmpalette(b).peBlue  : p += 1
          *p = bmpalette(b).peGreen : p += 1
          *p = bmpalette(b).peRed   : p += 1
        Next
      EndIf
   Else
   	ErLog("Only BMP files with 8 or 24 Bit Per Pixel at file "+Chr(34)+File+Chr(34)+" the file have "+Str(bitmapinfoheader.biBitCount))
      close #FF
      Return 0
   EndIf
   close #FF
   return ret
	
End Function
Function TEXTURE.LoadCompressedTGA ( ByVal File As String, ByRef W As Integer, ByRef H As Integer, ByRef BB As Integer ) As Any Ptr
	Type TTGAHeader
		header(0 To 5) As UByte
		BytePerPixel As UInteger
		Size As UInteger
		Height As UInteger
		width As UInteger
		Bpp As uinteger
	End Type
	
	Dim Header(11) As UByte
	Dim Image As UByte Ptr
	Dim As UByte cTGAcompare(11) => {0,0,10, 0,0,0,0,0,0,0,0,0}
	Dim FF As Integer
	FF = FreeFile
	If Open (File, For Binary, As FF) = 0 Then
		Get #FF, ,Header()
		If Not This.ArraySame(@Header(0),@cTGAcompare(0),12) Then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		Dim TGAHeader As TTGAHeader
		Get #FF, ,TGAHeader.header()
		W = TGAHeader.header(1) Shl 8 + TGAHeader.header(0)
		H = TGAHeader.header(3) Shl 8 + TGAHeader.header(2)
		BB = IIf ( TGAHeader.header(4) = 32, GL_RGBA, GL_RGB )
		TGAHeader.Width = W
		TGAHeader.Height = H
		TGAHeader.Bpp = TGAHeader.header(4) 
		if (TGAHeader.Width <= 0) or (TGAHeader.Height <= 0) or ((TGAHeader.Bpp <> 24) and (TGAHeader.Bpp <> 32)) Then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		TGAHeader.BytePerPixel = (TGAHeader.Bpp \ 8)
		TGAHeader.Size = (TGAHeader.BytePerPixel * TGAHeader.width * TGAHeader.Height)
		Image = Allocate(TGAHeader.Size)
		If Image = 0 then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		Dim As UInteger Pixel = (TGAHeader.width * TGAHeader.Height)
		Dim As UInteger currentpixel = 0
		Dim As UInteger currentbyte = 0
		Dim As UByte b
		Dim As UByte Ptr colorbuffer, pb
		colorbuffer = Allocate(TGAHeader.BytePerPixel)
		Do
			Dim As UByte chunkheader = 0
			Dim As Short counter = 0
			Get #FF,,chunkheader
			If chunkheader < 128 Then
				chunkheader += 1		
				While counter < chunkheader
					pb = colorbuffer
					For i As UByte = 0 to TGAHeader.BytePerPixel-1
						Get #FF, , b
						*pb = b : pb = pb + 1
					Next
					Image[currentbyte] = colorbuffer[2]
					Image[currentbyte + 1] = colorbuffer[1]
					Image[currentbyte + 2] = colorbuffer[0]
					If TGAHeader.BytePerPixel = 4 Then Image[currentbyte + 3] = colorbuffer[3]
					currentbyte += TGAHeader.BytePerPixel
					currentpixel += 1
					If currentpixel > pixel Then
						Close #FF
						If colorbuffer <> 0 Then DeAllocate ( colorbuffer )
						If Image <> 0 Then DeAllocate ( Image )
						W = 0
						H = 0
						BB = 0
						Return 0
					EndIf
					counter += 1
				Wend
			Else
				chunkheader -= 127
				pb = colorbuffer
				For i As UByte = 0 to TGAHeader.bytePerPixel-1
					Get #FF, , b
					*pb = b : pb = pb + 1
				Next
				counter = 0
				While counter < chunkheader
					Image[currentbyte] = colorbuffer[2]
					Image[currentbyte + 1] = colorbuffer[1]
					Image[currentbyte + 2] = colorbuffer[0]
					if TGAHeader.bytePerPixel = 4 Then Image[currentbyte + 3] = colorbuffer[3]
					currentbyte += TGAHeader.bytePerPixel
					currentpixel += 1
					if currentpixel > pixel Then
						Close #FF
						If colorbuffer <> 0 Then DeAllocate ( colorbuffer )
						If Image <> 0 Then DeAllocate ( Image )
						W = 0
						H = 0
						BB = 0
						Return 0
					EndIf
					counter += 1
				Wend
			EndIf
		Loop While ( currentpixel < pixel )
		Close #FF
		If colorbuffer Then DeAllocate ( colorbuffer )
		Return Image
	EndIf
	W = 0
	H = 0
	BB = 0
	Return 0
End Function
Function TEXTURE.LoadUncompressedTGA ( ByVal File As String, ByRef W As Integer, ByRef H As Integer, ByRef BB As Integer ) As Any Ptr
	Type TTGAHeader
		header(0 To 5) As UByte
		BytePerPixel As UInteger
		Size As UInteger
		Height As UInteger
		width As UInteger
		Bpp As uinteger
	End Type
	
	Dim Header(11) As UByte
	Dim Image As UByte Ptr
	Dim As UByte uTGAcompare(11) => {0,0,2, 0,0,0,0,0,0,0,0,0}
	Dim FF As Integer
	FF = FreeFile
	If Open ( File For Binary As FF ) = 0 Then
		Get #FF, ,Header()
		If Not This.ArraySame(@Header(0),@uTGAcompare(0),12) Then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		Dim TGAHeader As TTGAHeader
		Get #FF, ,TGAHeader.header()
		W = TGAHeader.header(1) Shl 8 + TGAHeader.header(0)
		H = TGAHeader.header(3) Shl 8 + TGAHeader.header(2)
		BB = IIf ( TGAHeader.header(4) = 32, GL_RGBA, GL_RGB )
		TGAHeader.Width = W
		TGAHeader.Height = H
		TGAHeader.Bpp = TGAHeader.header(4)
		if (TGAHeader.Width <= 0) or (TGAHeader.Height <= 0) or ((TGAHeader.Bpp <> 24) and (TGAHeader.Bpp <> 32)) Then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		TGAHeader.BytePerPixel = (TGAHeader.Bpp \ 8)
		TGAHeader.Size = (TGAHeader.BytePerPixel * TGAHeader.width * TGAHeader.Height)
		Image = Allocate(TGAHeader.Size)
		If Image = 0 then
			Close #FF
			W = 0
			H = 0
			BB = 0
			Return 0
		EndIf
		Dim As UByte r,g,b,a
		For i As UInteger = 0 to TGAHeader.Size - 1
			Get #FF, , b
			Get #FF, , g
			Get #FF, , r
			If TGAHeader.BytePerPixel = 4 Then
				Get #FF, , a
				Image[i*4+3] = a
			EndIf
			Image[i*TGAHeader.BytePerPixel] = r
			Image[i*TGAHeader.BytePerPixel+1] = g
			Image[i*TGAHeader.BytePerPixel+2] = b
		Next
		Close #FF
		Return Image
	EndIf
	W = 0
	H = 0
	BB = 0
	Return 0
End Function
Function TEXTURE.ArraySame ( ByVal Array1 As Any ptr, ByVal Array2 As Any Ptr, ByVal Leng As Long) As Long
	For i As UInteger = 0 To Leng-1
		If Cast ( UByte Ptr, Array1 )[i] <> Cast ( UByte Ptr, Array2 )[i] Then
			Return 0
		EndIf
	Next
	Return -1
End Function
Function TEXTURE.MaskPic ( ByVal Mask As UInteger, ByVal W As Integer, ByVal H As Integer, ByRef BBP As Integer, ByVal Z As Any Ptr ) As Any Ptr
	Dim ret As Any Ptr
	Dim As UByte R,G,B,A
	If Z = 0 Then Return 0
	If BBP <> GL_RGB Then Return Z
	ret = Allocate(W*H*4)
	If ret = 0 Then Return Z
	For i As Integer = 0 To W*H-1
		R = Cast(UByte Ptr,Z)[i*3]
		G = Cast(UByte Ptr,Z)[i*3+1]
		B = Cast(UByte Ptr,Z)[i*3+2]
		If RGB(R,G,B) = Mask Then
			R = 1
			G = 1
			B = 1
			A = 0
		Else
			A = 255
		EndIf
		Cast(UByte Ptr,ret)[i*4] = R
		Cast(UByte Ptr,ret)[i*4+1] = G
		Cast(UByte Ptr,ret)[i*4+2] = B
		Cast(UByte Ptr,ret)[i*4+3] = A
	Next
	DeAllocate(Z)
	BBP = GL_RGBA
	Return ret
End Function
