'GFXFrontPanel.bas - Game_207 front panel graphic creation
'Ryan Kelly (codeFoil) 2012

'==========================================================================|

#Include  once "Common.bi"

#IfDef  DEBUG_GFXFRONTPANEL
    #Define  DEBUG
#Endif
#Include  once "Debug.bi"

#Include  once "Pixel.bi"
#Include  once "GenGrain.bi"
#Include  once "SineWave.bi"
#Include  once "BrushWave.bi"
#Include  once "CarveWave.bi"
#Include  once "TraceWave.bi"
#Include  once "Layout.bi"

'==========================================================================|

public _
Sub  CraftFrontPanel(byval layer as Image_Buffer )

    DASSERT(layer <> NULL, "Null pointer!" )


    '---Inset-Holes-in-Grid-------------------------------------------------|   
    Scope

        For  rowIndex as Integer = 0 to BOARD_HEIGHT - 1 

            Dim  as Integer yPos = ROW_Y(rowIndex) + ROW_HEIGHT/2 _
                                 - FRONT_LAYER_Y
            
            For  slotIndex as Integer = 0 to BOARD_WIDTH - 1

                Dim  as Integer xPos = SLOT_X(slotIndex) + SLOT_WIDTH/2

              '---Draw Inset-----------------------------------------------|

                #Define INSET_COLOUR_DELTA (-FRONT_LAYER_HIGHLIGHT_DELTA/2)
                Dim  as unsigned Integer _
                insetColour = PixelFromDelta(FRONT_LAYER_COLOUR         ,_
                                             INSET_COLOUR_DELTA         ,_
                                             FRONT_LAYER_COLOUR_SCALES   )

                #Define  INSET_RADIUS (HOLE_RADIUS + 2 )
                
                Circle layer, (xPos, yPos), INSET_RADIUS, insetColour,,,,F

            Next  slotIndex

        Next  rowIndex

    End  scope

    '---Texture-Layer------------------------------------------------------|
    Scope

      'Create Wood Grain
    
        Dim  as Double grain(0 to FRONT_LAYER_HEIGHT - 1)
        
        GenGrain  grain(), FRONT_LAYER_GRAIN_DEPTH
    
        SineWave  grain(), FRONT_LAYER_FACE_CURVE_HEIGHT,_
                           FRONT_LAYER_FACE_CURVE_SCALE ,_
                           FRONT_LAYER_FACE_CURVE_PHASE
    
        BrushWaveLeftRight layer                    ,_
                           grain()                  ,_
                           FRONT_LAYER_COLOUR_SCALES

    End  scope

    '---Shape-Top-and-Bottom----------------------------------------------|
    Scope

        Dim  as Double curve(0 to FRONT_LAYER_WIDTH - 1 )

     'Carve-Top-Curve-----------------------------------------------------|

        SineWave  curve(), MARGIN_HORIZONTAL * 2/3                ,_
                           2 * ( GRID_WIDTH - 1 )               ,,,_
                           MARGIN_VERTICAL                        ,_
                           FRONT_LAYER_WIDTH - MARGIN_VERTICAL - 1
        
        CarveWaveLeftRight layer, curve(), 0, CARVE_DOWN

     'Bevel-Top-Edge------------------------------------------------------|

       For  fade as Integer = 1 to MARGIN_HORIZONTAL/4

            TraceWaveLeftRight layer, curve(), fade , TRACE_BELOW,_
                               FRONT_LAYER_HIGHLIGHT_DELTA / fade,_
                               FRONT_LAYER_COLOUR_SCALES

        Next  fade
        
     'Carve-Bottom-Curve--------------------------------------------------|
      
        SineWave  curve(), MARGIN_HORIZONTAL                      ,_
                           2 * ( GRID_WIDTH - 1 )               ,,,_
                           MARGIN_VERTICAL                        ,_
                           FRONT_LAYER_WIDTH - MARGIN_VERTICAL - 1,_
                           OP_SET
     
        SineWave  curve(), 1                                      ,_
                           2 * ( GRID_WIDTH - 1 )                 ,_
                           (-PI/2)                                ,_
                           PHI                                    ,_
                           MARGIN_VERTICAL                        ,_
                           FRONT_LAYER_WIDTH - MARGIN_VERTICAL - 1,_
                           OP_MUL                                 ,_
                           TRUE

        CarveWaveLeftRight  layer, curve(), FRONT_LAYER_HEIGHT - 1,_
                            CARVE_UP

      'Bevel-Bottom-Edge--------------------------------------------------|    

        For fade as Integer = 1 to MARGIN_HORIZONTAL/4    

           TraceWaveLeftRight layer, curve()                          ,_
                               FRONT_LAYER_HEIGHT - fade - 1          ,_
                               TRACE_ABOVE                            ,_
                               -2 * FRONT_LAYER_HIGHLIGHT_DELTA / fade,_
                               FRONT_LAYER_COLOUR_SCALES

        Next  fade

    End  scope

    '---Shape-Vertical-Side-----------------------------------------------|   
    Scope

        Dim  as Double curve(0 to FRONT_LAYER_HEIGHT - 1 )

      '---Carve-Vertical-Curves-------------------------------------------|

        SineWave  curve(), MARGIN_VERTICAL + 1                      ,_
                           2 * ( GRID_HEIGHT - 1 )                ,,,_
                           MARGIN_HORIZONTAL                        ,_
                           FRONT_LAYER_HEIGHT - MARGIN_HORIZONTAL - 1

        SineWave curve(),  1                                         ,_
                           2*( GRID_HEIGHT - 1 )                     ,_
                           (-PI/2)                                   ,_
                           PHI                                       ,_
                           MARGIN_HORIZONTAL                         ,_
                           FRONT_LAYER_HEIGHT - MARGIN_HORIZONTAL - 1,_
                           OP_MUL                                    ,_
                           TRUE

        CarveWaveUpDown  layer, curve(), 0, CARVE_RIGHT
        CarveWaveUpDown  layer, curve(), FRONT_LAYER_WIDTH - 1, CARVE_LEFT

      '---Bevel-Vertical-Edges---------------------------------------------|

        For  fade as Integer = 1 to MARGIN_VERTICAL/4

            TraceWaveUpDown  layer, curve(), fade, TRACE_RIGHT ,_
                             FRONT_LAYER_HIGHLIGHT_DELTA / fade,_
                             FRONT_LAYER_COLOUR_SCALES     
    
            TraceWaveUpDown  layer, curve()                         ,_
                             FRONT_LAYER_WIDTH - fade - 1           ,_
                             TRACE_LEFT                             ,_
                             -2 * FRONT_LAYER_HIGHLIGHT_DELTA / fade,_
                             FRONT_LAYER_COLOUR_SCALES

        Next  fade
                    
    End  scope

    '---Bore-Holes-in-Grid-------------------------------------------------|
    Scope

        For  rowIndex as Integer = 0 to BOARD_HEIGHT - 1 

            Dim  as Integer yPos = ROW_Y(rowIndex) + ROW_HEIGHT/2 _
                                 - FRONT_LAYER_Y
            
            For  slotIndex as Integer = 0 to BOARD_WIDTH - 1

                Dim  as Integer xPos = SLOT_X(slotIndex) + SLOT_WIDTH/2

              '---Draw Inset-----------------------------------------------|
                
                Circle layer, (xPos, yPos), HOLE_RADIUS,_
                       COLOUR_TRANSPARENT ,,,, F

            Next  slotIndex

        Next  rowIndex

    End  scope


End sub
