'Board_State.bas - Game 207 board logic
'Ryan Kelly (codeFoil) 2012

'===HEADER=FILES===========================================================|

#Ifdef  DEBUG_BOARD_STATE
    #Define  DEBUG   
#Endif
#Include  once "Debug.bi"

#Include  once "Board_State.bi"


'====ROUTINES==============================================================|


public _
Constructor  Board_State()

'Upon creation, simply reset game state to initial conditions

    This.Clear

End  constructor


public _
Sub  Board_State.Clear()

    For  s as Integer = 0 to BOARD_WIDTH - 1

        For  r as Integer  = 0 to BOARD_HEIGHT  - 1
            location( s , r ) = EMPTY_SPACE
        Next  r

        nextRow( s ) = 0 

    Next s
    
    turnNumber = -1
    currentPlayer = PLAYER_A
    movesRemaining = BOARD_WIDTH * BOARD_HEIGHT
    gameWinner = 0

End  sub


public _
Function  Board_State.PlaceToken(byval where as Integer) as Boolean

    DASSERT(where >= 0 AND where < BOARD_WIDTH   ,_
            "Invalid parameter: where = " & where )

    DASSERT(This.nextRow( where  ) < BOARD_HEIGHT     ,_
            "Invalid Move! Parameter where = " & where )


    location( where, nextRow( where ) ) = currentPlayer
    currentPlayer = -currentPlayer
    nextRow( where ) += 1
    turnNumber += 1
    moveHistory(turnNumber) = where
    movesRemaining -= 1

    function = CheckForWin()
'*****
'    For  rIndex as Integer = BOARD_HEIGHT - 1 to 0 Step -1
'        Dim  as String rString = ""
'        For  sIndex as Integer = 0 to BOARD_WIDTH - 1
'            Select  case location( sIndex, rIndex )
'            Case  PLAYER_A
'                rString &= "o"
'            Case  PLAYER_B
'                rString &= "x"
'            Case  else
'                rString &= " "
'            End  select
'        Next  sIndex
'        DPRINT(rString)
'    Next  rIndex
'    DPAUSE
End  function


public _
Function  Board_State.Token(byval querySlot as Integer,_
                            byval queryRow  as Integer ) as Integer

    DASSERT(querySlot >= 0 AndAlso querySlot < BOARD_WIDTH ,_
            "Invalid parameter querySlot: " & querySlot     )
    DASSERT(queryRow  >= 0 AndAlso queryRow  < BOARD_HEIGHT,_
            "Invaid parameter queryRow: "   & queryRow      )

    function = location( querySlot, queryRow )                 

End  function

public _
Function  Board_State.IsSlotOpen(byval querySlot as Integer ) as Boolean

    DASSERT(querySlot >= 0 AndAlso querySlot < BOARD_WIDTH ,_
    "Invalid parameter querySlot : " & querySlot            )

    function = FALSE
    If  nextRow(querySlot ) < BOARD_HEIGHT Then

        function = TRUE

    End  if

End  function


public _
Function  Board_State.NextRowInSlot(byval querySlot as Integer ) as Boolean

    DASSERT(querySlot >= 0 AndAlso querySlot < BOARD_WIDTH ,_
    "Invalid parameter querySlot : " & querySlot            )

    function = nextRow(querySlot )
    

End  function


public _
Sub  Board_State.Undo()

    If turnNumber >= 0  Then

        Dim as Integer lastMove = moveHistory(turnNumber)
        Dim as Integer lastRow = nextRow(lastMove) - 1
        location(lastMove, lastRow ) = EMPTY_SPACE
        nextRow(lastMove) -= 1
        turnNumber -= 1       
        movesRemaining += 1
        gameWinner = 0
        currentPlayer = -currentPlayer

    End  if

End  sub


public _
Property  Board_State.winner() as Integer

    property = gameWinner

End  property


public _
Property  Board_State.lastSlotPlayed() as Integer

    DASSERT(turnNumber >= 0, "Illegal call.  turnNumber = " & turnNumber )

    property = moveHistory( turnNumber )

End  property


public _
Property  Board_State.lastTurn() as Integer

    property = turnNumber

End  property

'--------------------------------------------------------------------------|

private _
Function  Board_State.CheckForWin() as Boolean

    DASSERT(turnNumber >= 0                     ,_
    "Game state invalid. turnNumber = " & turnNumber )

    function = FALSE

    Dim  as Integer lastMove = moveHistory(turnNumber)

    Dim  as Integer playedSlot = lastMove
    Dim  as Integer playedRow  = nextRow( playedSlot ) - 1
    Dim  as Integer player = location( playedSlot, playedRow )
    Dim  as Integer slotDelta( 0 to 3 ) => {-1, 0, 1, 1 },_
                     rowDelta( 0 to 3 ) => {-1,-1,-1, 0 }
    Dim  as Integer numConnected = 0,_
                    deltaIndex   = 0
    Do 
        #Macro  COUNT(_sS_,_sR_)
        CountConnected(player                     ,_
                       playedSlot                 ,_
                       playedRow                  ,_
                       _sS_ slotDelta(deltaIndex) ,_
                       _sR_  rowDelta(deltaIndex)  )
        #EndMacro
        numConnected = COUNT(0+,0+) + COUNT(0-,0-)
        #Undef  COUNT

        deltaIndex += 1
        
    Loop  while ( deltaIndex <= 3 ) AndAlso ( numConnected < 3 )    
    
    If  numConnected >= 3 Then
        gameWinner = player
        function = TRUE 
    ElseIf  movesRemaining = 0 Then 
        gameWinner = EMPTY_SPACE
        function = TRUE
    End  if

End  function


private _
Function  Board_State.CountConnected(byval player    as Integer,_
                                     byval initSlot  as Integer,_
                                     byval initRow   as Integer,_
                                     byval deltaSlot as Integer,_
                                     byval deltaRow  as Integer )_
                                     as Integer



    Dim  as Integer currentSlot = initSlot + deltaSlot
    Dim  as Integer currentRow  = initRow  + deltaRow
    Dim  as Integer count       = 0

    #Define  SLOT_IN_BOUNDS currentSlot >= 0 and currentSlot < BOARD_WIDTH 
    #Define  ROW_IN_BOUNDS  currentRow  >= 0 and currentRow  < BOARD_HEIGHT
    #Define  ISCONNECTED location( currentSlot, currentRow ) = player
    Do  while  SLOT_IN_BOUNDS AndAlso ROW_IN_BOUNDS AndAlso ISCONNECTED

        count       += 1
        currentSlot += deltaSlot
        currentRow  += deltaRow  

    Loop
    #Undef  SLOT_IN_BOUNDS
    #Undef  ROW_IN_BOUNDS
    #Undef  ISCONNECTED

    function = count

End  function