'Game_AI.bas - Game207 psuedo-inteligent oponent
'Ryan Kelly (codeFoil) 2012

'=========================================================================|

#IfDef DEBUG_GAME_AI
#Define DEBUG
#EndIf
#Include  once "Debug.bi"

#Include  once "Game_AI.bi"

'=========================================================================|
#IfDef  TRACE_AI_THREAD
#Define TPRINT(_x_) DPRINT("AI: " & _x_)
#Else
#Define TPRINT(_x_)
#EndIf

#Macro CHECK_INIT(_id_,_initFunc_)
    _id_ = _initFunc_
    If  _id_ = NULL Then  errorFlag = TRUE
#EndMacro

#Macro SAFE_DESTROY(_id_,_destSub_)
    If  _id_ Then  _destSub_ _id_
#EndMacro

'=========================================================================|

#Define AI_LOCK   MutexLock    aiThreadLock
#Define AI_UNLOCK MutexUnlock  aiThreadLock

'#Define DUMMY_AI

#Define MAX_WAIT_TIME 30

'=========================================================================|

public _
Constructor Game_AI()

    errorFlag  = FALSE
    trueState  = NULL
    mySide     = 0
    myMove     = 0

    exitSignal = FALSE
    gameSignal = FALSE
    moveSignal = FALSE
    playSignal = FALSE

    rootNode = NULL
    curNode  = NULL

    CHECK_INIT(aiThreadLock, MutexCreate())

    CHECK_INIT(condConnected  , CondCreate() )
    CHECK_INIT(condAcknowledge, CondCreate() )

    aiThreadHandle = NULL    
    If  Not errorFlag Then
        Dim  as any ptr threadEntry = @AIThreadEntry
        CHECK_INIT(aiThreadHandle, ThreadCreate(threadEntry, @This ) )
    End  if

End  constructor


public _
Destructor Game_AI()

    If      aiThreadHandle  _
    AndAlso aiThreadLock    _
    AndAlso condAcknowledge _
    AndAlso condConnected   _
    Then

        AI_LOCK
        exitSignal = TRUE
        If Not gameSignal Then  CondSignal condConnected
        CondWait  condAcknowledge, aiThreadLock
        If moveSignal Then CondSignal  condAcknowledge
        ThreadWait aiThreadHandle
        AI_UNLOCK
    End  if

    SAFE_DESTROY(aiThreadLock   , MutexDestroy )
    SAFE_DESTROY(condConnected  , CondDestroy  )
    SAFE_DESTROY(condAcknowledge, CondDestroy  )

    If rootNode <> NULL Then  Delete  rootNode

End  destructor

'=========================================================================|


public _
Sub  Game_AI.ConnectToGame(byval bState as Board_State ptr  ,_
                           byval side  as Integer           )

    DASSERT(bState <> NULL, "Null pointer! (bState)")
    DASSERT(side = PLAYER_A OrElse Side = PLAYER_B ,_
    "Invalid parameter Side : " & side              )

    trueState    = bState
    mySide       = side

    AI_LOCK
    gameSignal = TRUE
    If  playSignal = TRUE Then
        CondSignal condAcknowledge
    Else
        playSignal = FALSE
    End  if
    moveSignal = FALSE
    CondSignal  condConnected
    AI_UNLOCK
    
End  sub


public _
Sub  Game_AI.ReleaseGame()

    AI_LOCK
    gameSignal = FALSE
    moveSignal = FALSE
    AI_UNLOCK

End  sub


public _
Property  Game_AI.hasDecided() as Boolean

    DASSERT(trueState <> NULL        ,_
    "Object not properly initialized!")

    property = FALSE

    If  playSignal = TRUE Then

        TPRINT("I have chosen my move.")
        
        property = TRUE
        AI_LOCK
        moveSignal = FALSE
        CondSignal  condAcknowledge
        AI_UNLOCK

    ElseIf  moveSignal = FALSE  Then
            
        AI_LOCK
        timeOut = Timer + MAX_WAIT_TIME
        moveSignal = TRUE
        AI_UNLOCK

    End  if

End  property


public _
Property  Game_AI.move() as Integer

    DASSERT(trueState <> NULL        ,_
    "Object not properly initialized!") 

    TPRINT("I am playing my move.")

    AI_LOCK
    property = myMove
    AI_UNLOCK    


End  property

'=========================================================================|

private _
static  _
Sub  Game_AI.AIThreadEntry(byval me as pGame_AI )

    me->AIThread

End  sub


private _
Sub  Game_AI.AIThread()

    TPRINT("I am alive.")

    Do 
        Sleep THREAD_SLEEP_DURATION, 1
        TPRINT("I am awake.")

        Do until gameSignal 
            TPRINT("I am waiting to play.")
            AI_LOCK
            CondWait  condConnected, aiThreadLock
            AI_UNLOCK
            If  exitSignal Then  Exit  do, do
            BeginGame
        Loop

        Ponder

        If  moveSignal AndAlso Not exitSignal Then  ChooseMyMove


    Loop until exitSignal

    TPRINT("It was nice to be here.")    
    AI_LOCK
    CondSignal condAcknowledge
    AI_UNLOCK

End sub

'=========================================================================|

private _
Sub  Game_AI.BeginGame()

    TPRINT("I am in the game!")

    If rootNode <> NULL Then
        Delete  rootNode
    End  if

    Randomize Timer
    rootNode = New Tree_Node
    curNode  = rootNode
    ponderState.Clear
    
End  sub

'--------------------------------------------------------------------------|

private _
Sub  Game_AI.Ponder()

    TPRINT("I am considering options.")

    If rootNode = NULL  Then
        TPRINT("I have lost my look ahead tree!")
        Exit  Sub
    End  if
    Dim  as Boolean noOpenNodes

    Dim  as Integer turnNumber = ponderState.lastTurn
    Dim  as Integer maxLookAhead = 8
    lookAheadThreshold = 8
    curNode = rootNode
    lookAhead = 0
    depthSearched = 0
    noOpenNodes = TRUE

    Do
        If  curNode = rootNode Then noOpenNodes = TRUE
        If  lookAhead > depthSearched  Then
            depthSearched = lookAhead
        End  if

        Do  until   curNode->firstChild = NULL _
            OrElse  lookAhead >= maxLookAhead

            curNode = curNode->firstChild
            DASSERT(curNode <> NULL, "Null pointer. " & __LINE__ )
            ponderState.PlaceToken(curNode->slotTested )
            lookAhead += 1
            
        Loop

        If      curNode->openFlag          _
        AndAlso curNode->firstChild = NULL _
        AndAlso curNode->score >= -7       _
        Then
            For  slotIndex as Integer = 0 to BOARD_WIDTH - 1
                
                If  ponderState.IsSlotOpen(slotIndex ) Then
                    Dim  as pTree_Node newNode = New Tree_Node
                    If  newNode <> NULL  Then

                        newNode->slotTested = slotIndex
                        Dim  as Boolean _
                        testCase = ponderState.PlaceToken(slotIndex )
                        
                        If  testCase = TRUE Then
                            
                            curNode->openFlag = FALSE
                            
                            If  (ponderState.winner = mySide)  Then

                                newNode->score = 21
                                If  curNode->parent <> NULL _
                                Then curNode->parent->score += 2

                            ElseIf (ponderState.winner = -mySide) Then

                                curNode->score -= 2
                                If curNode->parent <> NULL Then
                                    Dim as pTree_Node p = curNode->parent
                                    If  p->parent <> NULL _
                                    Then  p->parent->score -= 3
                                End  if                              

                            End  if
                            
                        Else

                            newNode->openFlag = TRUE
                            noOpenNodes = FALSE

                        End  if

                        curNode->AddChild(newNode )

                        ponderState.Undo

                    End  if

                End  if

            Next  slotIndex

        End  if

        
        'ponderState.Undo
        Do  until   curNode->parent = NULL      _
            OrElse  curNode->nextSibling <> NULL 
            
            curNode = curNode->parent
            lookAhead -= 1
            ponderState.Undo

        Loop

        If  curNode->nextSibling <> NULL Then
            ponderState.Undo
            curNode = curNode->nextSibling
            ponderState.placeToken(curNode->slotTested)
        End  if
            

    Loop  until   (moveSignal  AndAlso depthSearched>=lookAheadthreshold)_
          OrElse  (depthSearched >= maxLookAhead )                       _
          OrElse  exitSignal                                             _
          OrElse  Not gameSignal                                         _
          OrElse  noOpenNodes    

    Do  until ponderState.lastTurn = turnNumber 
        ponderState.Undo
    Loop

End  sub


'--------------------------------------------------------------------------|

private _
Sub  Game_AI.ChooseMyMove()

    
    TPRINT("I am choosing my move." )

#IfDef  DUMMY_AI
    AI_LOCK
    myMove = RandomMove()
    playSignal = TRUE
    CondWait  condAcknowledge, aiThreadLock
    playSignal = FALSE
    AI_UNLOCK
#Else

    If  rootNode = NULL  Then

        TPRINT("I have a problem and must guess.")
        
        AI_LOCK
        myMove = RandomMove()        
        playSignal = TRUE
        CondWait  condAcknowledge, aiThreadLock
        playSignal = FALSE
        AI_UNLOCK

    Else
        
        If trueState->lastTurn > -1  Then
            curNode = rootNode->Find(trueState->lastSlotPlayed )
            ponderState.PlaceToken(curNode->slotTested )
        Else
            curNode = rootNode
        End  if

        Dim  as pTree_Node goodMoves( 0 to BOARD_WIDTH - 1 )
        Dim  as Integer gCount = 0
        Dim  as Integer bestScore = &h80000000

        Dim  as pTree_Node evalNode = curNode->firstChild
        Do  until evalNode = NULL
            If  evalNode->score > bestScore  Then
                TPRINT("Option score: " & evalNode->score )
                TPRINT("New best")
                bestScore = evalNode->score
                gCount = 0
            End  if
            If  evalNode->score = bestScore  Then
                TPRINT("Option score: " & evalNode->score )
                TPRINT("Save score")
                goodMoves( gCount ) = evalNode
                gCount += 1
                TPRINT(Str(gCount))
            End  if
            evalNode = evalNode->nextSibling
        Loop
        DPRINT(Str(gCount))
        evalNode = NULL
        
        AI_LOCK
        If gCount > 0  Then
            evalNode = goodMoves( Int(Rnd() * gCount ) )
            myMove = evalNode->slotTested
        Else
            myMove = RandomMove()

        End  if
        playSignal = TRUE
        CondWait  condAcknowledge, aiThreadLock
        playSignal = FALSE
        AI_UNLOCK

        ponderState.PlaceToken(myMove )

        If evalNode <> NULL  Then
            evalNode->Remove()
            Delete rootNode
            rootNode = evalNode            
        Else

            delete rootNode
            rootNode = New Tree_Node

        End  If
        
    End  if

#EndIf

End  sub


private _
Function  Game_AI.RandomMove() as Integer

    Dim  as Integer rMove
    Do
        rMove = Int(Rnd * BOARD_WIDTH )
    Loop until trueState->IsSlotOpen(rMove)
    function = rMove

End  function



'=========================================================================|

