'Bounding_Box.bi - Game_207  Bounding Box Object
'Ryan Kelly (codeFoil) 2012

'==========================================================================|

#IfDef DEBUG_BOUNDING_BOX
    #Define DEBUG
#EndIf
#Include  Once "Debug.bi"

#Include  Once "GUI\Bounding_Box.bi"

'==========================================================================|

public _ 
Constructor  Bounding_Box()

    Set 0, 0, 0, 0

End  constructor


public _
Function  Bounding_Box.Set(byval newX as Integer,_
                           byval newY as Integer,_
                           byval newW as Integer,_
                           byval newH as Integer ) as Boolean

    DASSERT( newW >= 0, "Invalid parameter. ( newW = " & newW & " )" )
    DASSERT( newH >= 0, "Invalid parameter. ( newH = " & newH & " )" )
    

    x = newX
    maxX = Iif(newW <> 0, newX + newW - 1, newX )
    width     = newW

    y = newY
    maxY = Iif(newH <> 0, newY + newH - 1, newY )
    height    = newH

    emptyFlag = FALSE
    If  maxX = x OrElse maxY = y then  emptyFlag = TRUE

    function = emptyFlag
    
End  function


public _
function  Bounding_Box.MoveTo(byval newX as Integer,_
                              byval newY as Integer ) as Boolean

    function = Set(newX, newY, width, height )

End  function


public _
function  Bounding_Box.Resize(byval newW as Integer,_
                              byval newH as Integer ) as Boolean

    function = Set(x, y, newW, newH )

End  function


public _
Function  Bounding_Box.GetAll(byref outX as Integer,_
                              byref outY as Integer,_
                              byref outW as Integer,_
                              byref outH as Integer ) as Boolean

    outX = x
    outY = y
    outW = width
    outH = height
    function = emptyFlag

End  function


public _
Function  Bounding_Box.GetFirst(byref outX as Integer,_
                                byref outY as Integer ) as Boolean

    outX = x
    outY = y
    function = emptyFlag

End  function


public _
Function  Bounding_Box.GetSize(byref outW as Integer,_
                               byref outH as Integer ) as Boolean

    outW = width
    outH = height
    function = emptyFlag

End  function


public _
Function  Bounding_Box.GetLast(byref outX as Integer,_
                               byref outY as Integer ) as Boolean

    outX = maxX
    outY = maxY
    function = emptyFlag

End  function                               


public _ 
Function  Bounding_Box.DoesEnclose(byval testX as Integer,_
                                   byval testY as Integer ) as Boolean

    function = TRUE

    If      x > testX _
    OrElse  testX     >  maxX  _
    OrElse  y > testY _
    OrElse  testY     >  maxY  _
    Then

        function = FALSE

    End  if

End  function

public _
Function  Bounding_Box.Draw(byval dest   as Image_Buffer,_
                            byval colour as Colour_Value,_
                            byval mode   as Box_Mode     _
                                          = BOX_OUTLINE  ) as Boolean

     DASSERT(dest <> NULL,_#######################
    "Null pointer passed as destination buffer!")

    If  emptyFlag = TRUE Then

        function = FALSE

    Else
                
        If mode = BOX_FILLED  Then
            Line dest, (x,y)-(maxX,maxY), colour, BF
        ElseIf mode = BOX_OUTLINE Then
            Line dest, (x,y)-(maxX,maxY), colour, B
        Else
            DPRINT ("Parameter invalid. mode = " & mode)
        End if
        function = TRUE

    End if
    

End  function


public _
Function  Bounding_Box.Etch(byval dest      as Image_Buffer ,_
                            byval colour    as Colour_Value ,_
                            byval mode      as Box_Etch_Mode,_
                            byval thickness as Integer = 1   ) as Boolean
                            
    DASSERT( dest <> NULL,_######################
    "Null Pointer passed as destination buffer!")

    function = emptyFlag

    If  thickness > 0 AndAlso Not emptyFlag  Then

        Dim   as Integer halfWidth  = width / 2
        Dim   as Integer halfHeight = height / 2
        thickness = MIN_OF(thickness, halfWidth  )
        thickness = MIN_OF(thickness, halfHeight )
            
        Dim  as Integer count = 0

        Do  until count > thickness 

            If  TEST_OPTION(mode, BOX_ETCH_LEFT) Then
                Line  dest,_
                      ( x + count , y +    count ) _
                     -( x + count , maxY - count ),_
                      colour 
            End  if

            If  TEST_OPTION(mode, BOX_ETCH_TOP ) Then
                Line  dest,_
                      ( x +    count, y + count ) _
                     -( maxX - count, y + count ),_
                      colour 
            End if

            If  TEST_OPTION(mode, BOX_ETCH_BOTTOM ) Then
                Line  dest,_
                      ( x +     count, maxY - count ) _
                     -( maxX  - count, maxY - count ),_
                      colour 
            End  if

            If  TEST_OPTION(mode, BOX_ETCH_RIGHT )  Then
                Line  dest,_
                      ( maxX - count, y +    count ) _
                     -( maxX - count, maxY - count ),_
                      colour 
            End  if

            count += 1
    
        Loop
        
    End  if

End  function
                            
public _ 
Function  Bounding_Box.DrawEllipse(byval dest   as Image_Buffer,_
                                   byval colour as Colour_Value,_
                                   byval mode   as Ellipse_Mode )_
                                   as Boolean

   DASSERT(dest <> NULL,_#######################
   "Null pointer passed as destination buffer!")
   If  emptyFlag = TRUE Then

        function = FALSE

    Else

        Dim  as Integer centerX = x + ((width-1)/2) ,_
                        centerY = y + ((height-1)/2),_
                        radius  = ((width)/2) 
        Dim  as Double  aspect  = height/width
                    
        If mode = ELLIPSE_FILLED  Then
            Circle dest, (centerX,centerY), radius,colour, , ,aspect, F
        ElseIf mode = ELLIPSE_OUTLINE Then
            Circle dest, (centerX,centerY), radius,colour, , ,aspect
        Else
            DPRINT ("Parameter invalid. mode = " & mode)
        End if
        function = TRUE

    End if

End  function

'-------------------------------------------------------------------------|

public _ 
Property  Bounding_Box.isEmpty() as Boolean

    property = emptyFlag

End  property

#Macro ACCESS_PROPERTY(_Pid_,_Mid_)
public _
Property Bounding_Box._Pid_() as Integer

    property = (_Mid_)

End  property
#EndMacro

ACCESS_PROPERTY(x1, x    )
ACCESS_PROPERTY(x2, maxX  )
ACCESS_PROPERTY(y1, y     )
ACCESS_PROPERTY(y2, maxY  )
ACCESS_PROPERTY( w, width )
ACCESS_PROPERTY( h, height)

#Macro SET_PROPERTY(_Pid_,_Exp_)
public _
Property  Bounding_Box._Pid_(byval newVal as Integer)
    _Exp_
End property
#EndMacro

SET_PROPERTY(x1, x = newVal )
SET_PROPERTY(x2, (Set(x, y, newVal - x + 1, height)) )
SET_PROPERTY(y1, y = newVal )
SET_PROPERTY(y2, (Set(x, y, width , newVal - y +1 )) )
SET_PROPERTY( w, (Set(x, y, newVal, height )) )
SET_PROPERTY( h, (Set(x, y, width , newVal )) )


                                 