'Button.bi - Game_207 Button Control
'Ryan Kelly (codeFoil) 2012

'==========================================================================|

#IfDef  DEBUG_BUTTON_CONTROL
    #Define  DEBUG
#Endif
#Include  once "Debug.bi"

#Include  once "GUI\Button_Control.bi"

'==========================================================================|

public _
Constructor  Button_Control(byval initStyle as BUTTON_STYLE          ,_
                            byval initX     as Integer               ,_
                            byval initY     as Integer               ,_
                            byval initW     as Integer               ,_
                            byval initH     as Integer               ,_
                            byval fColour   as Colour_Value = DEF_BFC,_
                            byval hColour   as Colour_Value = DEF_BHC,_
                            byval sColour   as Colour_Value = DEF_BSC,_
                            byval image     as Image_Buffer = NULL   ,_
                            byval imageInv  as Image_Buffer = NULL    )

    errorFlag = FALSE
    style = initStyle

    box.Set initX, initY, initW, initH

    faceColour       = fColour
    highlightColour  = hColour
    workingHighlight = highlightColour
    shadowColour     = sColour
    workingShadow    = shadowColour

    faceImage       = image
    faceInverted    = image
    ownFaceImage    = FALSE
    ownFaceInverted = FALSE

    visibleFlag   = TRUE
    enabledFlag    = TRUE
    depressedFlag = FALSE

    mouseIsMine = FALSE

    toolTipText = ""
    showToolTip = FALSE

    If  TEST_OPTION(style,BUTTON_IMAGE) AndAlso faceImage = NULL Then
        faceImage = ImageCreate(initW, initH, fColour )
        If  faceImage = NULL Then 
            errorFlag = TRUE 
        Else
            ownFaceImage = TRUE
        End  if
    End  if

    If  TEST_OPTION(style,BUTTON_IMAGE) AndAlso faceInverted = NULL Then
        faceInverted = ImageCreate(initW, initH, sColour )
        If  faceInverted = NULL Then
            errorFlag = TRUE
        Else
            ownFaceImage = TRUE
        End  if
    End  if        

End  constructor


public _
Destructor  Button_Control()

    If ownFaceImage AndAlso faceImage <> NULL Then
        ImageDestroy  faceImage
    End  if

    If ownFaceInverted AndAlso faceInverted <> NULL Then
        ImageDestroy  faceInverted

    End if
        

End  destructor

'--------------------------------------------------------------------------|
#Macro  EXPOSE_BOOLEAN(_type_,_member_,_Set_,_Clear_,_access_)

public _
Function  _type_._Set_() as Boolean

    function    = _member_
    _member_ = TRUE

End  function

public _
Function  _type_._Clear_() as Boolean

    function    = _member_
    _member_ = FALSE

End  function

public _
Property  _type_._access_() as Boolean

    property = _member_

End  property

#EndMacro

EXPOSE_BOOLEAN(Button_Control,enabledFlag  ,Enable,  Disable, isEnabled  )
EXPOSE_BOOLEAN(Button_Control,visibleFlag  ,Show  ,  Hide   , isVisible  )

'EXPOSE_BOOLEAN(Button_Control,depressedFlag,PushDown,PullUp, isDepressed )

public _
Function  Button_Control.PushDown() as Boolean

    function = depressedFlag
    depressedFlag = TRUE
    workingHighlight = shadowColour
    workingShadow    = highLightColour

End  function

public _ 
Function  Button_Control.PullUp() as Boolean

    function = depressedFlag
    depressedFlag = FALSE
    workingHighlight = highlightColour
    workingShadow = shadowColour

End  function

public _
Property  Button_Control.isDepressed() as Boolean

    property = depressedFlag

End  property
'--------------------------------------------------------------------------|

public _
Function  Button_Control.Toggle() as Boolean

    function = depressedFlag
    depressedFlag = Not depressedFlag

    Swap  workingHighlight, workingShadow    
    
End  function



'--------------------------------------------------------------------------|

public _ 
Sub  Button_Control.Fill(byval fillRoutine as Image_Crafter )

    DASSERT(fillRoutine <> NULL, "Null pointer!")
    fillRoutine(faceImage)

End  sub

public _ 
Sub  Button_Control.FillInverted(byval fillRoutine as Image_Crafter )

    DASSERT(fillRoutine <> NULL, "Null pointer!")
    fillRoutine(faceInverted)

End  sub


public _
Function  Button_Control.Draw(byval dest as Image_Buffer) as Boolean

    DASSERT(dest<> NULL, "Null Pointer!")


    function = FALSE

    If  visibleFlag  Then

        If  TEST_OPTION(style, BUTTON_CIRCLE) Then
            function = DrawCircleButton(dest )
        Else
            function = DrawRectangularButton(dest )
        End if     

        If  TEST_OPTION(style, BUTTON_IMAGE) Then

            Dim  as Integer iX, iY
            box.GetFirst(iX,iY)
            
            If      depressedFlag                          _
            AndAlso TEST_OPTION(style, BUTTON_IMAGE_INVERT)_
            Then
                Put dest, (iX,iY), faceInverted, TRANS
            Else
                Put dest, (iX,iY), faceImage, TRANS
            End  if
        End  if

        If  showToolTip Then

            Dim  as Integer tX, tY
            box.GetLast tX,tY
            .Draw String dest, (tX,tY), toolTipText, workingHighlight

        End if

    End  if 

End  function


private  _
Function _
Button_Control.DrawRectangularButton(dest as Image_Buffer) as Boolean

                                               
    If  Not TEST_OPTION(style, BUTTON_IMAGE)  Then
        box.Draw(dest, faceColour, BOX_FILLED )
    End  if

    If  TEST_OPTION(style, BUTTON_RAISED )  Then
        box.Etch(dest             ,_
                 workingHighlight ,_
                 BOX_ETCH_TOP_LEFT,_
                 DEF_BBVL          )
        box.Etch(dest                 ,_
                 workingShadow        ,_
                 BOX_ETCH_RIGHT_BOTTOM,_
                 DEF_BBVL              )
    End  if

    If  TEST_OPTION(style, BUTTON_EXIT )  Then

        Dim  as Integer x1, x2, y1, y2, w, h

        box.GetFirst(x1, y1 )
        box.GetLast( x2, y2 )
        box.GetSize( w , h  )

        x1 = x1 + (w/4)
        x2 = x2 - (w/4)
        y1 = y1 + (h/4)
        y2 = y2 - (h/4)

        Line dest, (x1,y1) - (x2-1,y2), workingShadow
        Line dest, (x1+1,y1) - (x2,y2), workingHighlight

        Line dest, (x1,y2) - (x2-1,y1), workingShadow
        Line dest, (x1+1,y2) - (x2,y1), workingHighlight

    End if

    function = TRUE
                      
End  function

private _
Function  Button_Control.DrawCircleButton(byval dest as Image_Buffer)_
                                         as Boolean
    Dim  as Integer x,y,w,h
    box.GetAll x, y, w, h
    If  TEST_OPTION(style, BUTTON_RAISED )  Then
        box.DrawEllipse(dest, workingShadow, ELLIPSE_FILLED)
        box.MoveTo x,y-2
    End  if

    If  Not TEST_OPTION(style, BUTTON_IMAGE) Then
        box.DrawEllipse(dest, faceColour, ELLIPSE_FILLED )
        box.DrawEllipse(dest, workingHighlight, ELLIPSE_OUTLINE)

    End  if
    box.Set x,y,w,h

    

    function = TRUE

End  function



'--------------------------------------------------------------------------|
public   _
static   _
Function _
Button_Control.HandleEvent(byval this as Button_Control ptr,_
                           byref event as Screen_Event      ) as Boolean

    DASSERT(this <> NULL, "Null Pointer!")
    function = FALSE

    If this->enabledFlag  Then
        Select  case event.code
        Case  MOUSE_BUTTON_PRESS
            If       event.mouseButton = MOUSE_BUTTON_LEFT _
            AndAlso  this->mouseIsMine Then
                this->PushDown
                function = TRUE
            End  if
        Case  MOUSE_BUTTON_RELEASE
            If  event.mouseButton = MOUSE_BUTTON_LEFT Then
                If       this->PullUp            _
                AndAlso  this->mouseIsMine       _
                Then
                     function = this->actionClick.Fire()                        
                End  if
            End  if
        Case MOUSE_MOVE
            this->mouseIsMine = this->box.doesEnclose(event.mouseX,_
                                                      event.mouseY )
            If       this->mouseIsMine _
            AndAlso  Len(this->toolTipText)  _
            Then

                this->showToolTip = TRUE

            Else

                this->showToolTip = FALSE    
            
            End  if 
             
        End Select        

    End  if

End  function