'Game_Window.bas - Game207 top level control
'Ryan Kelly (codeFoil) 2012

'==========================================================================|

#IfDef  DEBUG_GAME_WINDOW
#Define DEBUG
#Endif
#Include  once "Debug.bi"

#Include  once "GUI\Game_Window.bi"

'==========================================================================|

Declare  sub CraftFrontPanel(byval layer as Image_Buffer )

Declare  sub CraftBackPanel(byval layer as Image_Buffer )

Declare  sub CraftToken(byval layer as Image_Buffer )

Declare  sub CraftIndicator(byval layer as Image_Buffer )

Declare  function BlendFade(byval src as Colour_Value,_
                            byval dst as Colour_Value,_
                            byval lvl as any ptr      ) as Colour_Value

'==========================================================================|

public _
Constructor  Game_Window()

    errorFlag   = frontLayer.errorFlag
    errorFlag Or= backLayer.errorFlag
    errorFlag Or= playerAButton.errorFlag
    errorFlag Or= playerBButton.errorFlag
    errorFlag Or= slotSelector.errorFlag
    errorFlag Or= tokenSprite.errorFlag
    errorFlag Or= gameAi.errorFlag

    imageBuffer = CreateImage(WINDOW_WIDTH      ,_
                              WINDOW_HEIGHT     ,_
                              COLOUR_TRANSPARENT )

    imageTokenA = CreateImage(PLAYER_TOKEN_WIDTH   ,_
                              PLAYER_TOKEN_HEIGHT  ,_
                              PLAYER_TOKEN_COLOUR_A,_
                              @CraftToken           )

    imageTokenB = CreateImage(PLAYER_TOKEN_WIDTH   ,_
                              PLAYER_TOKEN_HEIGHT  ,_
                              PLAYER_TOKEN_COLOUR_B,_
                              @CraftToken           )

    imageIndicator = CreateImage(PLAYER_TOKEN_WIDTH ,_
                                 PLAYER_TOKEN_HEIGHT,_
                                 COLOUR_TRANSPARENT ,_
                                 @CraftIndicator     )

    If  Not errorFlag Then

        frontLayer.Fill(@CraftFrontPanel)

        backLayer.Fill(@CraftBackPanel)

        playerAButton.Fill(@CraftToken)
        playerBButton.Fill(@CraftToken)

        boardDisplay.SetImages imageTokenA, imageTokenB
        boardDisplay.ConnectState(@boardState)


        InitHandler(EXIT_BUTTON, @exitButton, @Button_Control.HandleEvent )
        exitButton.actionClick.Set(EXIT_BUTTON, @This, @ActionExit )

        InitHandler(RESET_BUTTON, @ResetButton, @Button_Control.HandleEvent )
        resetButton.actionClick.Set(RESET_BUTTON,@This, @ActionReset)

        InitHandler(PLAYER_A_BUTTON           ,_
                   @playerAButton             ,_
                   @Button_Control.HandleEvent )
        playerAButton.actionClick.Set(PLAYER_A_BUTTON  ,_
                                      @this            ,_
                                      @ActionSelectSide )
       
        InitHandler(PLAYER_B_BUTTON           ,_
                   @playerBButton             ,_
                   @Button_Control.HandleEvent )
        playerBButton.actionClick.Set(PLAYER_B_BUTTON ,_
                                      @this           ,_
                                      @ActionSelectSide)

        slotSelector.ConnectState(@boardState )
        InitHandler(SLOT_CONTROL              ,_
                    @slotSelector             ,_
                    @Slot_Selector.HandleEvent )
        slotSelector.actionClick.Set(SLOT_CONTROL     ,_
                                     @this            ,_
                                     @ActionPlaceToken )

        tokenSprite.targetReached.Set(NULL            ,_
                                      @this           ,_
                                      @ActionPlaceToken)

        indicator.SetImage imageIndicator, 0
        indicator.Hide

        resetButton.toolTipText = "Clear Board"

    End  if

    ScreenControl SC_GET_WINDOW_POS, myX, myY
                          
End  constructor


private _
Sub  Game_Window.InitHandler(byval index  as Game_Window_Control_Index,_
                             byval newOb  as any ptr                  ,_
                             byval newHR  as any ptr                   )

    control(index).object = newOb
    control(index).handler = Cast(Event_Handler_Routine, newHr)

End  sub


public _
Destructor  Game_Window()

    If  imageBuffer <>  NULL Then
        ImageDestroy  imageBuffer
    End  if
    If  imageTokenA <> NULL Then
        ImageDestroy  imageTokenA
    End  if
    If  imageTokenB <> NULL Then
        ImageDestroy  imageTokenB
    End  if
    If  imageIndicator <> NULL Then
        ImageDestroy  imageIndicator
    End  if

End  destructor


private _
Function  Game_Window.CreateImage(byval imW as Integer           ,_
                                  byval imH as Integer           ,_
                                  byval imC as Colour_Value       _
                                             = COLOUR_TRANSPARENT,_
                                  byval crR as Image_Crafter      _
                                             = NULL               )_
                                  as Image_Buffer

    DASSERT(imW > 0, "Invalid parameter imW : " & imW )
    DASSERT(imH > 0, "Invalid parameter imH : " & imW )

    Dim as Image_Buffer newIm = ImageCreate(imW, imH, imC )

    If  newIm = NULL Then
        DPRINT("Image Creation failed!")
        errorFlag = TRUE
    ElseIf crR <> NULL Then
        crR(newIM)
    End  if

    function = newIm

End  function

'--------------------------------------------------------------------------|

public _
Sub  Game_Window.SetMode(byval newMode as Game_Window_Mode )

    Dim  as Game_Window_Mode oldMode = mode

    mode = newMode

    Select  case mode
    Case  MODE_PLAYER_SELECTION

        gameAI.ReleaseGame

        boardState.Clear
        
        resetButton.Enable
        resetButton.Show

        playerAButton.Enable
        playerAButton.Show

        playerBButton.Enable
        playerBButton.Show

        slotSelector.Disable
        slotSelector.Hide

        tokenSprite.Hide
        indicator.Hide

        textBox.SetText("Click a Token to Select Color")
        textBox.Show

    Case MODE_GET_TURN

        playerAButton.Disable

        playerBButton.Disable

        textBox.Hide

        StartTurn

    Case MODE_ANIMATE_UP

        slotSelector.Disable
        slotSelector.Hide

    Case MODE_ANIMATE_DOWN

    Case MODE_GAME_OVER

        gameAi.ReleaseGame

        indicator.Hide

        If  winCondition = EMPTY_SPACE  Then

            textBox.SetText("Draw")

        ElseIf  winCondition = playerSide  Then

            textBox.SetText("You Win!")

        Else

            textBox.SetText("Game Over") 

        End if

        textBox.Show

    Case Else

        mode = oldMode

    End  select
    
End  sub

'--------------------------------------------------------------------------|

public _
Function  Game_Window.HandleInput() as Boolean

    function = TRUE

    Dim  as Screen_Event event

    Do  while ScreenEvent(@event)

        Dim  as Boolean eventHandled = FALSE

        Dim  as Game_WIndow_Control_Index index = 0

        Do until   trackMouse            _
           OrElse  index = CONTROL_COUNT _
           OrElse  eventHandled         

            Dim  as any ptr object = control(index).object

            DASSERT(control(index).handler <> NULL,_###########
            "Handler not set for control index " & index & ".")

            eventHandled = control(index).handler(object, event)

            index += 1

        Loop

        If  Not eventHandled  Then

            Select case event.code
            Case MOUSE_MOVE

                mouseX = event.mouseX
                mouseY = event.mouseY

                If  trackMouse Then MoveTo(myX + event.mouseDX,_
                                           myY + event.mouseDY )

            Case MOUSE_BUTTON_PRESS

                If  event.mouseButton = MOUSE_BUTTON_LEFT Then
                    trackMouse = TRUE
                End  if

            Case MOUSE_BUTTON_RELEASE

                If  event.mouseButton = MOUSE_BUTTON_LEFT Then
                    trackMouse = FALSE
                End  if

            Case WINDOW_CLOSE

                exitFlag = TRUE

            End select

        End  if

    Loop

    If      mode =  MODE_GET_TURN   _
    AndAlso playerSide <> whoseTurn _
    AndAlso gameAi.hasDecided       _
    Then

            slotSelector.selectedSlot = gameAi.move

            BeginTokenPlacement

    End  if

    If this.exitFlag = TRUE Then function = FALSE

End  function

'--------------------------------------------------------------------------|
private _
Sub  Game_Window.StartGame(byval userSide as Integer )

    playerSide = userSide

    whoseTurn  = PLAYER_A

    gameAi.ConnectToGame @boardState                                  ,_
                         Iif(playerSide = PLAYER_A, PLAYER_B, PLAYER_A )

    SetMode MODE_GET_TURN

End  sub


private _ 
Sub  Game_Window.StartTurn()

   slotSelector.SetColor Iif(whoseTurn  = PLAYER_A,_
                             PLAYER_TOKEN_COLOUR_A,_
                             PLAYER_TOKEN_COLOUR_B )

    If  whoseTurn = playerSide  Then

        indicator.Hide
        slotSelector.Show
        slotSelector.Enable

    Else

        slotSelector.Disable

        Dim  as Integer iX = Iif(whoseTurn = PLAYER_A,_
                                 PLAYER_TOKEN_A_X    ,_
                                 PLAYER_TOKEN_B_X     )
        indicator.box.MoveTo iX, PLAYER_TOKEN_Y
        indicator.BeginPulse 2, 0, 60  


    End  if

    

End  sub


private _
Sub  Game_Window.BeginTokenPlacement()

    Static as Integer slotX, slotSelected

    Select  case mode
    Case  MODE_GET_TURN

        slotSelected = slotSelector.selectedSlot

        slotX = SLOT_X(slotSelected )

        Dim  as Image_Buffer tokenImage
        If whoseTurn = PLAYER_A  Then

            tokenImage = imageTokenA

        Else

            tokenImage = imageTokenB

        End  if
        tokenSprite.SetImage  tokenImage, 0

        Dim  as Integer startX = Iif(whoseTurn = PLAYER_A,_
                                 PLAYER_TOKEN_A_X        ,_
                                 PLAYER_TOKEN_B_X         )

        Dim  as Integer startY = PLAYER_TOKEN_Y

        tokenSprite.box.MoveTo(startX, startY )

        tokenSprite.Target slotX, 0, 10 

        SetMode(MODE_ANIMATE_UP)

    Case  MODE_ANIMATE_UP

        Dim  as Integer _
        slotY = ROW_Y(boardState.NextRowInSlot(slotSelected ) )

        tokenSprite.Target slotX, slotY, 10, TRUE

        SetMode(MODE_ANIMATE_DOWN )

    Case  MODE_ANIMATE_DOWN

        tokenSprite.Hide()

        If  boardState.PlaceToken( slotSelected ) Then

            winCondition = boardState.Winner
            SetMode MODE_GAME_OVER

        Else

            whoseTurn = -whoseTurn

            SetMode MODE_GET_TURN

        End if

    Case  Else

        DPRINT("Unknown Mode in BeginTokenPlacement")

    End Select

End  sub

'--------------------------------------------------------------------------|

public _
Sub  Game_Window.MoveTo(byval newX as Integer,_
                        byval newY as Integer )

    myX = newX
    myY = newY

    ScreenControl SC_SET_WINDOW_POS, myX, myY

End  sub

public _
Sub  Game_Window.Center()

    Dim  as Integer deskW, deskH

    ScreenControl SC_GET_DESKTOP_SIZE, deskW, deskH

    MoveTo (deskW - myW)/2, (deskH - myH)/2

End  sub


public _ 
Sub Game_Window.FadeIn()

    If  imageBuffer <> NULL Then

        Compose

        For  level as Integer = 35 to 255 Step 20

            ScreenSync
            ScreenLock
            Put (0,0), imageBuffer, CUSTOM, @BlendFade, @level
            ScreenUnlock

        Next  level

    End  if
       
End  sub

public _ 
Sub Game_Window.FadeOut()

    If  imageBuffer <> NULL Then

        Compose

        For  level as Integer = 255 to 135 Step -20

            ScreenSync
            ScreenLock
            Put (0,0), imageBuffer, CUSTOM, @BlendFade, @level
            ScreenUnlock

        Next  level

    End  if
       
End  sub


public _
Sub  Game_Window.Draw()

    If  imageBuffer <> NULL Then
        
        Compose
        ScreenLock
        Put  (0,0), imageBuffer, PSET
        ScreenUnlock

    End  if

End  sub

public _
Sub  Game_Window.Compose()

        Line  imageBuffer, (0,0)-(WINDOW_WIDTH - 1, WINDOW_HEIGHT - 1 ),_
              COLOUR_TRANSPARENT, BF
              
        backLayer.Draw(imageBuffer )
        tokenSprite.Draw(imageBuffer, TRUE)
        slotSelector.Draw(imageBuffer)
        boardDisplay.Draw(imageBuffer )
        frontLayer.Draw(imageBuffer)
        playerAButton.Draw(imageBuffer)
        playerBButton.Draw(imageBuffer)
        exitButton.Draw(imageBuffer)
        resetButton.Draw(imageBuffer)
        tokenSprite.Draw(imageBuffer, FALSE)
        textBox.Draw(imageBuffer)
        indicator.Draw(imageBuffer)
        

End  sub

'--------------------------------------------------------------------------|


private _
Function Game_Window.ActionExit(byval this as Game_Window_ptr,_
                                byval id   as Integer         ) as Boolean

    this->exitFlag = TRUE
    function = TRUE

End function

private _
Function Game_Window.ActionReset(byval this as Game_Window_ptr,_
                                 byval id   as Integer         ) as Boolean

    this->SetMode MODE_PLAYER_SELECTION 
    function = TRUE

End function


private _
Function Game_Window.ActionSelectSide(byval this as Game_Window_ptr,_
                                      byval id   as Integer         )_
                                      as Boolean

    this->StartGame Iif(id = PLAYER_A_BUTTON,_
                        PLAYER_A            ,_
                        PLAYER_B             )
    function = TRUE
    
End  function

private _
Function  Game_Window.ActionPlaceToken(byval this as Game_Window_ptr,_
                                       byval id   as Integer         )_
                                       as Boolean

    this->BeginTokenPlacement
    function = TRUE

End  function