'Sprite.bas -  Game207 animated sprite
'Ryan Kelly (codeFoil) 2012
'==========================================================================|

#IfDef  DEBUG_SPRITE
#Define  DEBUG
#EndIf
#Include  once "Debug.bi"

#Include  once "GUI\Sprite.bi"

'==========================================================================|

public _
Constructor  Sprite(byval initX as Integer     ,_
                    byval initY as Integer     ,_
                    byval initW as Integer     ,_
                    byval initH as Integer     ,_
                    byval numF  as Integer  = 1 )

    
    

    errorFlag = FALSE

    box.Set(initX, initY, initW, initH )

    frameCount = numF
    currentFrame = 0
    currentAlpha = 255
    minAlpha     = 1
    maxAlpha     = 255
    deltaX       = 0
    deltaY       = 0

    frame = New Image_Buffer[frameCount]
    frameIsMine = New Boolean[frameCount]

    If frame = NULL Then 

        DPRINT("Sprite constructor failed to allocation frame array!")
        errorFlag = TRUE
        
    Else

        For  fIndex as Integer = 0 to frameCount - 1

            frame[fIndex] = NULL

        Next fIndex

    End  if

    If frameIsMine = NULL Then

        DPRINT("Sprite constructor failed to allocation ownership array!")
        errorFlag = TRUE

    Else
    
        For  fIndex as Integer = 0 to frameCount -1

            frameIsMine[fIndex] = FALSE

        Next fIndex
    
    End if

    animateFlag    = FALSE
    visibleFlag    = FALSE
    backGroundFlag = FALSE
    resetFlag      = FALSE
    restartFlag    = FALSE
    pulseFlag      = FALSE
    strobeFlag     = FALSE
    movingFlag     = FALSE
    

End  constructor


public _
Destructor  Sprite()

   If  frame <> NULL Then

        For  fIndex as Integer = 0 to frameCount

            If       frameIsMine <> NULL        _
            AndAlso  frameIsMine[fIndex] = TRUE _
            Then
                ImageDestroy frame[fIndex]
            End  if

        Next  fIndex

        Delete[]  frame

   End  if

    If frameIsMine <> NULL Then  Delete[]  frameIsMine

End  destructor


public _
Sub  Sprite.SetImage(byval newImg as Image_Buffer,_
                     byval fIndex as Integer      )

    DASSERT(newImg <> NULL, "Null pointer!")

    DASSERT(fIndex <=0 AndAlso fIndex < frameCount,_
    "Invalid parameter : fIndex="     & fIndex    &_
                       " frameCount=" & frameCount )

    DASSERT(frame<>NULL, "Sprite did not initialize!")
    DASSERT(frameIsMine<>NULL, "Sprite did not initialize!")

    If frame[fIndex] <> NULL AndAlso frameIsMine[fIndex]  Then
        ImageDestroy  frame[fIndex]
    End  if

    

    frame[fIndex] = newImg
    frameIsMine[fIndex] = FALSE

    
End  sub
'--------------------------------------------------------------------------|

public _
Property  Sprite.isVisible() as Boolean

    property = visibleFlag

End  property


public _
Function  Sprite.Show() as Boolean

    function = visibleFlag

    visibleFlag = TRUE

End  function

public _
Function  Sprite.Hide() as Boolean

    function = visibleFlag

    visibleFlag = FALSE

End  function


'--------------------------------------------------------------------------|

public _ 
Sub  Sprite.Draw(byval dest             as Image_Buffer,_
                 byval drawOnBackGround as Boolean      ) 

    DASSERT(dest<> NULL, "Null pointer!")
    DASSERT(frame<>NULL, "Sprite failed to initialize!")

   If  visibleFlag AndAlso backgroundFlag = drawonBackGround Then
        Dim  as Integer x, y
        box.GetFirst x, y 
        If      currentFrame < frameCount   _
        AndAlso frame[currentFrame] <> NULL _
        Then
            Put dest, (x,y), frame[currentFrame], ALPHA, currentAlpha
        End  if
        If  animateFlag  Then
            currentFrame += 1
            If       currentFrame = frameCount _
            AndAlso  resetFlag                 _
            Then
                currentFrame = 0
                animateFlag = restartFlag
            Else
                visibleFlag = FALSE
            End  if
        End  if
        If      pulseFlag  _
        OrElse  strobeFlag _
        Then
            currentAlpha += alphaDelta 
            If  currentAlpha > maxAlpha  Then
                If  strobeFlag Then
                    currentAlpha = minAlpha
                Else
                    currentAlpha = maxAlpha
                    alphaDelta = -alphaDelta
                End if
            Elseif currentAlpha < minAlpha Then
                currentAlpha = minAlpha
                alphaDelta = -alphaDelta
            End  if
        End  if
        If movingFlag  Then
            Dim  as Integer signBefore = Sgn(curX-targetX)
            curX += deltaX
            If  signBefore <> Sgn(curX-targetX)  Then
                curX = targetX
                deltaX = 0
            End  iF
            signBefore = Sgn(curY-targetY)
            curY += deltaY
            If signBefore <> Sgn(curY-targetY)  Then
                curY = targetY
                deltaY = 0
            End  If

            If      deltaX = 0 _
            AndAlso deltaY = 0 _
            Then
                movingFlag = 0
                targetReached.Fire
            End  if
            box.MoveTo(Int(curX),Int(curY))
        End  if
    Endif

End  sub

'--------------------------------------------------------------------------|

public _
Sub  Sprite.BeginAnimation(byval fReset      as Boolean  = TRUE,_
                           byval fRestart    as Boolean = FALSE,_
                           byval fBackground as Boolean = FALSE )

    resetFlag      = fReset
    restartFlag    = fRestart
    backGroundFlag = fBackGround
    visibleFlag    = TRUE
    animateFlag    = TRUE
    currentFrame   = 0

End  sub

public _
Sub  Sprite.Target(byval tX          as Integer        ,_
                   byval tY          as Integer        ,_
                   byval nSteps      as Integer        ,_
                   byval fBackground as Boolean = FALSE )

    DASSERT(nSteps > 0                     ,_
    "Invalid parameter: nsteps = " & nsteps )

    backGroundFlag = fBackGround

    Dim as Integer x, y
    box.GetFirst x, y

    curX = x
    curY = y

    Dim as Double diffX    = tX - curX,_
                  diffY    = tY - curY
    deltaX   = diffX/nsteps
    deltaY   = diffY/nsteps
   
    targetX = tX
    targetY = tY

    movingFlag = TRUE
    visibleFlag = TRUE    
    
End  sub

public _
Sub  Sprite.BeginStrobe(byval aDelta as Integer = 1 ,_
                        byval aMin   as Integer = 1 ,_
                        byval aMax   as Integer = 255)

    visibleFlag  = TRUE
    strobeFlag   = TRUE
    minAlpha     = aMin
    maxAlpha     = aMax
    alphaDelta   = aDelta
    currentAlpha = aMin

End  sub

public _
Sub  Sprite.BeginPulse(byval aDelta as Integer = 1   ,_
                       byval aMin   as Integer = 1   ,_
                       byval aMax   as Integer = 255  )
    
    visibleFlag  = TRUE
    pulseFlag    = TRUE
    minAlpha     = aMin
    maxAlpha     = aMax
    alphaDelta   = aDelta
    currentAlpha = aMax    

End  sub

'==========================================================================|
