'Game_Window.bi - Game_207 graphical user Interface main control
'Ryna Kelly (codeFoil) 2012

'==========================================================================|

'Included by GUI.bi
'Include order must be preserved!

#IfnDef  GAME207_GUI
#Include  once "Common.bi"
#Include  once "Screen_Event.bi"
#Include  once "Bounding_Box.bi"
#Include  once "Action.bi"
#Include  once "Sprite.bi"
#Include  once "Text_Message.bi"
#Include  once "Static_Image.bi"
#Include  once "Button_Control.bi"
#Include  once "Slot_Selector.bi"
#Include  once "Board_Display.bi"
#EndIf
#Include  once "Game_AI.bi"

'===GAME=WINDOW============================================================|

Enum  Game_Window_Mode
    MODE_PLAYER_SELECTION
    MODE_GET_TURN
    MODE_GAME_OVER
    MODE_ANIMATE_UP
    MODE_ANIMATE_DOWN
End  enum


Enum Game_Window_Control_Index
    EXIT_BUTTON = 0
    RESET_BUTTON 
    SLOT_CONTROL
    PLAYER_A_BUTTON
    PLAYER_B_BUTTON
    CONTROL_COUNT 
End  enum

Type  Event_Handler_Routine _
as Function(byval as any ptr, byref as Screen_Event ) as Boolean

Type Event_Handler
    as any ptr object
    as Event_Handler_Routine handler
End  type
    
Type Game_Window_ptr as Game_Window ptr
Type  Game_Window

  Public:
    Declare  constructor()
    Declare  destructor()

  Private:
    Declare  sub InitHandler(byval index as Game_Window_Control_Index,_
                             byval newOb as any ptr                  ,_
                             byval newHR as any ptr                   )

    Declare  function CreateImage(byval imW as Integer           ,_
                                  byval imH as Integer           ,_
                                  byval imC as Colour_Value       _
                                             = COLOUR_TRANSPARENT,_
                                  byval crR as Image_Crafter      _
                                             = NULL               )_
                                  as Image_Buffer

  Public:
    Declare  sub SetMode(byval newMode as Game_Window_Mode )

    Declare  function HandleInput() as Boolean

    Declare  sub  FadeIn()
    Declare  sub  FadeOut()
    Declare  sub  Draw()
    Declare  sub  Compose()

    Declare  sub  MoveTo(byval newX as Integer,_
                         byval newY as Integer )

    Declare  sub  Center()

    as Boolean errorFlag = FALSE,_
               exitFlag  = FALSE

  Private:

    Declare  sub StartGame(byval userSide as Integer )
    Declare  sub StartTurn()
    Declare  sub BeginTokenPlacement()


    as Game_Window_Mode mode
    as Game_AI     gameAI
    as Board_State boardState
    as Integer playerSide
    as Integer whoseTurn
    as Integer     winCondition
    
    as Image_Buffer imageBuffer   ,_
                    imageTokenA   ,_
                    imageTokenB   ,_
                    imageIndicator

    as Integer myX, myY

    as Integer  myW = WINDOW_WIDTH,_
                myH = WINDOW_HEIGHT

    as Integer mouseX, mouseY
    as Boolean trackMouse = FALSE

    as Event_Handler control(0 to CONTROL_COUNT -1 )


    as Slot_Selector slotSelector
    as Board_Display boardDisplay
    
    as Text_Message textBox = Text_Message _
                              ("INIT"                    ,_
                               TEXT_MESSAGE_FILL_COLOUR  ,_
                               TEXT_MESSAGE_BORDER_COLOUR,_
                               TEXT_MESSAGE_TEXT_COLOUR   )

    as Button_Control exitButton = Button_Control                _
                                   (BUTTON_EXIT Or BUTTON_RAISED,_
                                    EXIT_BUTTON_X               ,_
                                    EXIT_BUTTON_Y               ,_
                                    EXIT_BUTTON_WIDTH           ,_
                                    EXIT_BUTTON_HEIGHT          ,_
                                    EXIT_BUTTON_COLOUR          ,_
                                    EXIT_BUTTON_HIGHLIGHT       ,_
                                    EXIT_BUTTON_SHADOW           )


    as Button_Control resetButton = Button_Control         _
                                   (RESET_BUTTON_STYLE    ,_
                                    RESET_BUTTON_X        ,_
                                    RESET_BUTTON_Y        ,_
                                    RESET_BUTTON_WIDTH    ,_
                                    RESET_BUTTON_HEIGHT   ,_
                                    RESET_BUTTON_COLOUR   ,_
                                    RESET_BUTTON_HIGHLIGHT,_
                                    RESET_BUTTON_SHADOW    )



    as Button_Control playerAButton = Button_Control            _
                                      (BUTTON_CIRCLE Or         _   
                                       BUTTON_RAISED Or         _
                                       BUTTON_IMAGE            ,_               
                                       PLAYER_TOKEN_A_X        ,_
                                       PLAYER_TOKEN_Y          ,_
                                       PLAYER_TOKEN_WIDTH      ,_
                                       PLAYER_TOKEN_HEIGHT     ,_
                                       PLAYER_TOKEN_COLOUR_A   ,_
                                       PLAYER_TOKEN_HIGHLIGHT_A,_
                                       PLAYER_TOKEN_SHADOW_A    )

    as Button_Control playerBButton = Button_Control            _
                                      (BUTTON_CIRCLE Or         _   
                                       BUTTON_RAISED Or         _
                                       BUTTON_IMAGE            ,_               
                                       PLAYER_TOKEN_B_X        ,_
                                       PLAYER_TOKEN_Y          ,_
                                       PLAYER_TOKEN_WIDTH      ,_
                                       PLAYER_TOKEN_HEIGHT     ,_
                                       PLAYER_TOKEN_COLOUR_B   ,_
                                       PLAYER_TOKEN_HIGHLIGHT_B,_
                                       PLAYER_TOKEN_SHADOW_B    )

    as Static_Image frontLayer = Static_Image _
                                 (FRONT_LAYER_X    , FRONT_LAYER_Y     ,_
                                  FRONT_LAYER_WIDTH, FRONT_LAYER_HEIGHT,_
                                  FRONT_LAYER_COLOUR                    )

    as Static_Image backLayer = Static_Image _
                                (BACK_LAYER_X    , BACK_LAYER_Y     ,_
                                 BACK_LAYER_WIDTH, BACK_LAYER_HEIGHT,_
                                 BACK_LAYER_COLOUR                   )

    as Sprite tokenSprite = Sprite _
                            (0                 , 0                  ,_
                             PLAYER_TOKEN_WIDTH, PLAYER_TOKEN_HEIGHT,_
                             1                                       )
    as Sprite indicator = Sprite _
                          (0                 , 0                  ,_
                           PLAYER_TOKEN_WIDTH, PLAYER_TOKEN_HEIGHT,_
                           1                                       )
                                  

    Declare  static function ActionExit(byval this as Game_Window_ptr,_
                                        byval id   as Integer        )_
                                        as Boolean

    Declare  static function ActionReset(byval this as Game_Window_ptr,_
                                         byval id   as Integer        )_
                                         as Boolean

    Declare  static function ActionSelectSide _
                                       (byval this as Game_Window_ptr,_
                                        byval id   as Integer        )_
                                        as Boolean

    Declare  static function ActionPlaceToken _
                                       (byval this as Game_Window_ptr,_
                                        byval id   as Integer         )_
                                        as Boolean

End  type